/*
 * -------------------------------------------------------------------------
 * DBell's user-mode pipeline library.
 * Copyright (C) 2001  David I. Bell (dbell@canb.auug.org.au)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * -------------------------------------------------------------------------
 *
 * The compress pseudo-device.
 * Currently this is just a wrapper for using the process device to
 * run compress.  The actual code could be inserted here - someday.
 */

#include "upPrivate.h"


/*
 * Add the "compress" pseudo-device.
 * It accepts a number of bits of compression code.
 * The value zero means use the default number of bits.
 */
UP_ID
upAddCompress(UP * up, int bits)
{
	char	cmd[80];

	if ((bits != 0) && ((bits < 0) || (bits > 16)))
	{
		upError(up, UP_ERR_BADCOMPRESSBITS);

		return UP_NULL_ID;
	}

	if (bits > 0)
		sprintf(cmd, "compress -f -c -b %d", bits);
	else
		strcpy(cmd, "compress -f -c");

	return upAddProcessString(up, cmd, UP_FLAG_PROC_SYSTEM_PATH, NULL);
}
