/*
 * -------------------------------------------------------------------------
 * DBell's user-mode pipeline library.
 * Copyright (C) 2001  David I. Bell (dbell@canb.auug.org.au)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * -------------------------------------------------------------------------
 *
 * The full-data device.
 * This device enforces full buffers by repeatedly doing reading or writing
 * until the specified amount of data is fully processed.
 */

#include "upPrivate.h"


static	UP_BOOL	doReadWrite(UP_DEV *);
static	int	doread(UP_DEV *, char *, int);
static	int	dowrite(UP_DEV *, const char *, int);


static UP_FUNC	funcs =
{
	UP_TYPE_FULL,
	"Full",
	UP_FALSE,
	UP_FALSE,
	UP_FALSE,
	UP_FALSE,
	UP_TRUE,
	UP_TRUE,
	upDefaultPollTable,
	doReadWrite,
	upDefaultFlush,
	upDefaultClose,
	upDefaultReset
};


UP_ID
upAddFull(UP * up)
{
	UP_DEV *	dev;

	dev = upAllocDevice(up, &funcs, 0);

	if (dev == UP_NULL_DEV)
		return UP_NULL_ID;

	return upLinkDevice(up, dev);
}


static UP_BOOL
doReadWrite(UP_DEV * dev)
{
	return UP_TRUE;
}


static int
doread(UP_DEV * dev, char * buf, int len)
{
	return upFullRead(dev->next, buf, len);
}


static int
dowrite(UP_DEV * dev, const char * buf, int len)
{
	return upFullWrite(dev->next, buf, len);
}
