/*
 * -------------------------------------------------------------------------
 * DBell's user-mode pipeline library.
 * Copyright (C) 2001  David I. Bell (dbell@canb.auug.org.au)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * -------------------------------------------------------------------------
 *
 * The reader endpoint device.
 * This device is provided as a support for the upRead routine.
 * All input data just sits in the input buffer until the upRead
 * routine is explicitly called to collect it.  This ignores all
 * file boundaries and just concatenates the files together.
 */

#include <unistd.h>

#include "upPrivate.h"


static	UP_BOOL	doBuildPollTable(UP_DEV *);
static	UP_BOOL	doReadWrite(UP_DEV *);


static UP_FUNC	funcs =
{
	UP_TYPE_READ_ENDPOINT,
	"ReadEndpoint",
	UP_FALSE,
	UP_TRUE,
	UP_FALSE,
	UP_FALSE,
	UP_FALSE,
	UP_TRUE,
	doBuildPollTable,
	doReadWrite,
	upDefaultFlush,
	upDefaultClose,
	upDefaultReset
};


UP_ID
upAddReadEndpoint(UP * up)
{
	UP_DEV *	dev;

	dev = upAllocDevice(up, &funcs, 0);

	if (dev == UP_NULL_DEV)
		return UP_NULL_ID;

	return upLinkDevice(up, dev);
}


static UP_BOOL
doBuildPollTable(UP_DEV * dev)
{
	dev->pollCount = 0;
	dev->pollTimeout = -1;

	return UP_TRUE;
}


static UP_BOOL
doReadWrite(UP_DEV * dev)
{
	UP_BUF *	readBuffer;
	UP_ITEM_TYPE	itemType;

	readBuffer = dev->readBuffer;

	/*
	 * Get the type of item which is available.
	 */
	itemType = upBufferGetReadItemType(dev->readBuffer);

	/*
	 * Handle the item.
	 */
	switch (itemType)
	{
		case UP_ITEM_TYPE_NONE:
		case UP_ITEM_TYPE_FILE_DATA:
			return UP_TRUE;

		case UP_ITEM_TYPE_FILE_NAME:
			upBufferReadFileName(readBuffer);

			break;

		case UP_ITEM_TYPE_END_OF_FILE:
			upBufferReadEof(readBuffer);

			break;

		case UP_ITEM_TYPE_END_OF_BUFFER:
			return upSetFinished(dev);

		case UP_ITEM_TYPE_FILE_ERROR:
		case UP_ITEM_TYPE_FATAL_ERROR:
			upDeviceError(dev, UP_ERR_BUFFER);

			return UP_FALSE;
		default:
			UP_DEVICE_INTERNAL_ERROR(dev);

			return UP_FALSE;
	}

	/*
	 * Return saying we have done I/O.
	 */
	return upSetDidReadWrite(dev);
}
