/*
 * -------------------------------------------------------------------------
 * DBell's user-mode pipeline library.
 * Copyright (C) 2001  David I. Bell (dbell@canb.auug.org.au)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * -------------------------------------------------------------------------
 *
 * The null reading endpoint device.
 * This simply returns an end of buffer.
 */

#include "upPrivate.h"


static	UP_BOOL	doBuildPollTable(UP_DEV *);
static	UP_BOOL	doReadWrite(UP_DEV *);


static UP_FUNC	funcs =
{
	UP_TYPE_READ_NULL,
	"ReadNull",
	UP_TRUE,
	UP_FALSE,
	UP_FALSE,
	UP_TRUE,
	UP_FALSE,
	UP_TRUE,
	doBuildPollTable,
	doReadWrite,
	upDefaultFlush,
	upDefaultClose,
	upDefaultReset
};


UP_ID
upAddReadNull(UP * up)
{
	UP_DEV *	dev;

	dev = upAllocDevice(up, & funcs, 0);

	if (dev == UP_NULL_ID)
		return UP_NULL_ID;

	return upLinkDevice(up, dev);
}


static UP_BOOL
doBuildPollTable(UP_DEV * dev)
{
	/*
	 * We want to run right away.
	 */
	dev->pollCount = 0;
	dev->pollTimeout = 0;

	return UP_TRUE;
}


static UP_BOOL
doReadWrite(UP_DEV * dev)
{
	/*
	 * Mark the output buffer as having final data and mark ourself
	 * as being finished.
	 */
	upBufferWriteEndOfBuffer(dev->writeBuffer);

	return upSetFinished(dev);
}
