/*
 * -------------------------------------------------------------------------
 * DBell's user-mode pipeline library.
 * Copyright (C) 2001  David I. Bell (dbell@canb.auug.org.au)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * -------------------------------------------------------------------------
 *
 * The null writing endpoint device.
 * This simply consumes all input and produces no output.
 */

#include "upPrivate.h"


static	UP_BOOL	doReadWrite(UP_DEV *);


static UP_FUNC	funcs =
{
	UP_TYPE_WRITE_NULL,
	"WriteNull",
	UP_FALSE,
	UP_TRUE,
	UP_FALSE,
	UP_FALSE,
	UP_FALSE,
	UP_TRUE,
	upDefaultBuildPollTable,
	doReadWrite,
	upDefaultFlush,
	upDefaultClose,
	upDefaultReset
};


UP_ID
upAddWriteNull(UP * up)
{
	UP_DEV *	dev;

	dev = upAllocDevice(up, & funcs, 0);

	if (dev == UP_NULL_ID)
		return UP_NULL_ID;

	return upLinkDevice(up, dev);
}


static UP_BOOL
doReadWrite(UP_DEV * dev)
{
	UP_BUF *	readBuffer;

	readBuffer = dev->readBuffer;

	/*
	 * Switch on the state of the input buffer.
	 * Acknowledge what we must, and ignore what we can.
	 */
	switch (upBufferGetReadItemType(readBuffer))
	{
		case UP_ITEM_TYPE_NONE:
			return UP_TRUE;

		case UP_ITEM_TYPE_FILE_NAME:
			upBufferReadFileName(readBuffer);

			break;

		case UP_ITEM_TYPE_FILE_DATA:
			upBufferReadClear(readBuffer);

			break;

		case UP_ITEM_TYPE_END_OF_FILE:
			upBufferReadEof(readBuffer);

			break;

		case UP_ITEM_TYPE_FILE_ERROR:
			upBufferReadFileError(readBuffer);

			break;

		case UP_ITEM_TYPE_END_OF_BUFFER:
			upBufferReadEndOfBuffer(readBuffer);

			return upSetFinished(dev);

		case UP_ITEM_TYPE_FATAL_ERROR:
			upDeviceError(dev, UP_ERR_BUFFER);

			return UP_FALSE;

		default:
			upDeviceError(dev, UP_ERR_INTERNAL);

			return UP_FALSE;
	}

	/*
	 * We successfully did something.
	 * Say that we have done I/O this cycle.
	 */
	return upSetDidReadWrite(dev);
}
